'==============================================================
'	ANALOGUE LED CLOCK v1.1 
'	PIC 16F877 @ 20MHz 
'
'	By D.J.Welburn July 2010 
'	djwprojects@live.co.uk  
'	www.youtube.com/user/wellyb00t
'	http://twitter.com/wellyb00t
'
'	Updated 17 May 2011 - Less flicker - No thrills version
'	
'
'	169 x 3mm white 700mcd leds 
'	PicBasicPro 2.46 - MPlab 8.15A
'		
'==============================================================	 

eeprom 1,[1,3,5,7,9,11,13,15,17,19,21,23]
eeprom 13,[23,23,0,0,0,1,1,2,2,2,3,3,4,4,4,5,5,6,6,6,7,7,8,8,8,9,9,10,10,10,11]
eeprom 44,[11,12,12,12,13,13,14,14,14,15,15,16,16,16,17,17,18,18,18,19]
eeprom 64,[19,20,20,20,21,21,22,22,22]

@	__config 0x3F76	'HS xtal
DEFINE OSC 20

TRISA = %00010110		'RA5 - B11 Output
TRISB = 0			'B0-7 outputs ULN2003 I/P
TRISD = 0			'White outputs RD0 = OUTER RING
TRISC = 0			'Green outputs RC0 = OUTER RING
TRISE = 0			'B8,9,10 Outputs to ULN2003 I/p
'-------------------------------------------------------------------------------------------------
'----------------------------I2C Bus SETUP for DS1307---------------------------------------------
'-------------------------------------------------------------------------------------------------

cont	var	byte 
addr	var	byte 
ICD	var	PORTA.3	'I2C DATA
ICC	var	PORTA.4	'I2C CLOCK
RTCsec	var	byte	'seconds in BCD 
RTCmin	var	byte 	'Minutes in BCD
RTChour	var	byte	'Hours in BCD
hour	var	byte	'Hours in DEC
minu	var	byte 	'Minutes in DEC

'-------------------------------------------------------------------------------------------------
'----------------------------Buttons Setup--------------------------------------------------------
'-------------------------------------------------------------------------------------------------
SYMBOL set = PORTA.2	'Set button
SYMBOL move = PORTA.1	'+ Button
'-------------------------------------------------------------------------------------------------
'----------------------------Varibles ------------------------------------------------------------
'-------------------------------------------------------------------------------------------------

hourhand	var	byte 		'LED Hour Hand 
minhand		var	byte		'LED mins Hand 
minuchange	var	byte		'For checking if mins as changed 
setup		var	byte		'setup flag 1=mins 2=hours setup 0=not setting up	
rowA		VAR	BYTE [24]	'LED DATA ARRAY
scancol		VAR	BYTE		'For scaning led array 
scan		VAR	WORD		'for scanning led array
counter		var	word		'General purpose counter WORD
counterB	var	byte		'as above
counterC	var	byte		'As above
pivot		var	bit		'pivot led
rtcbit		var	byte		'Counter for DS1307 read

CLEAR				'Clear all varibles to 0
cont = %11010000		'I2C control $D0
ADCON1 = %00000111		'all port A digital i/o
'----------------------------------------------------------------------------------
'----------------------- Set Up timer interupts------------------------------------
'----------------------------------------------------------------------------------
OPTION_REG = %00000110	'1:128 prescale int every 6.5ms
ON INTERRUPT GOTO display
INTCON = %10100000
'----------------------------------------------------------------------------------
'----------------------- Start up LED display demo --------------------------------
'----------------------------------------------------------------------------------
MAIN:

minuchange = 61
pivot = 1
FOR counter = 0 TO 4500
	PAUSEUS 5
NEXT counter
pivot = 0 
FOR counter = 0 TO 4500
	PAUSEUS 5
NEXT counter
pivot = 1
'==================================================================================================
'============================= UPDATING HAND POSITION =============================================
'==================================================================================================	
loop:					
	IF minu <> minuchange THEN		'When minute changes 
		FOR counter = 0 TO 23		'The display clears
			rowA [counter] = 0
		NEXT
		READ hour, hourhand			'Hourhand position is read from eeprom
		IF minu > 31 THEN			'If minutes is greater than 31 then
			IF hourhand = 23 THEN		'hourhand position changes to the next postion 
				hourhand = 0		'midway between 2 hour postions
			ELSE
				hourhand = hourhand + 1
			ENDIF
		ENDIF
		rowA [hourhand] = %01111000		'Defines the LED Hourhand
		READ minu+13, minhand			'minute hand position is read from eeprom
		rowA [minhand] = %01111111		'defines LED minute hand
		IF minhand = hourhand THEN		'If both min and hour on same hand, 3rd LED goes out
			rowA [minhand] = rowA [minhand] - 4
		ENDIF
		minuchange = minu			'minuchange & minu will remain the same value until
	ENDIF						'the minute changes 

'=================================================================================================
'===================================Time Setting==================================================
'=================================================================================================
setloop:
	rowA.0(95) = pivot 	'centre led	
	IF set = 0 THEN
		pivot = 0
		setup = setup + 1 			
		IF setup > 2 THEN
			INTCON.7 = 0 	'disable interupts
			RTCmin = minu DIG 1 		'-----------------------
			RTCmin = RTCmin << 4 		' Covert decimal to BCD |     
			RTCmin = RTCmin + minu DIG 0	'-----------------------
			RTChour = hour DIG 1	
			RTChour = RTChour << 4 		
			RTChour = RTChour + hour DIG 0 
			RTChour.6 = 1				'Set Ds1307 into 12hr mode
			I2CWrite ICD, ICC, cont, addr, [0, RTCmin, RTChour,1,1,1,1,0]	'Write data in BCD to DS1307
			PAUSE 10
			INTCON.7 = 1	'enable interupts
			setup = 0
			GOTO MAIN
		ENDIF
		FOR counter = 0 TO 5000	'Pause Routine for button debounce
			PAUSEUS 5
		NEXT counter	
	ENDIF
	IF move = 0 AND setup = 1 THEN	' === SET MINUTE HAND POSITION ===
		rowA [minhand] = 0
		minu = minu + 1
		IF minu > 59 THEN
			minu = 0
		ENDIF
		READ minu+13, minhand		'Reads new Min Hand position
		FOR counter = 0 TO 4500
			PAUSEUS 5
		NEXT counter	
	ENDIF
	IF move = 0 AND setup = 2 THEN	' === SET HOUR HAND POSITION ===
		rowA [hourhand] = 0
		hour = hour + 1
		IF hour > 12 THEN
			hour = 1
		ENDIF
		READ hour, hourhand		'Reads new hour hand position
	
		FOR counter = 0 TO 4500
			PAUSEUS 5
		NEXT counter	
	ENDIF	
	IF setup <> 0 THEN
		rowA [hourhand] = %01111000		'------------------------------	
		rowA [minhand] = %01111111		'Update Hand Poitions on clock|
		IF hourhand = minhand THEN		'
			rowA [hourhand] = %01111011	'
		ENDIF					'------------------------------
		GOTO setloop
	ENDIF
GOTO loop

'============================================================================================
'===================================Display update routine===================================
'============================================================================================
DISABLE
display:
	
	PORTB = %10101010	'<--------------------------------------
	PORTA.5 = 1		'<--					|
	PORTE.1 = 1		'<--					|
	PORTC = %00000001	'<--					|
	PORTD = %00000001	'<-This bit provides dim outer edge LEDs|
	PAUSEUS 80		'<--					|
	PORTC = 0		'<--					|
	PORTD = 0		'<--------------------------------------|
	
	rtcbit = rtcbit + 1
	IF setup = 0 AND rtcbit = 100 THEN						'Reads DS1307 every 100 interupts 
		rtcbit = 0
		I2CRead ICD, ICC, cont, addr, [RTCsec, RTCmin, RTChour]			'Reads Ds1307 via I2c bus
		RTCHour.6 = 0: RTChour.5 = 0						'Clears 12hr mode & AM/PM marker bits
		hour = RTChour >> 4:hour = hour * 10:RTChour = RTChour &$0F: hour = hour + RTChour 	'Convert BCD to decimal for hours (&$0F clears upper 4bits)
		minu = RTCmin >> 4:minu = minu * 10: RTCmin = RTCmin &$0F: minu = minu + RTCmin 	'Convert BCD to decimal for mins
	ENDIF
	'======================SCAN THE DISPLAY 1 ROW (HAND) AT A TIME, EACH LIT FOR 500us====================================
	scan = 1									
	FOR scancol = 0 TO 11								
		PORTB = scan.lowbyte			'PIN 33 = B0 & PIN 40 = B7        
		PORTE.2 = scan.8 			'Pin 10 - B8						
		PORTE.1 = scan.9 			'Pin 9  - B9		
		PORTE.0 = scan.10  			'Pin 8  - B10
		PORTA.5 = scan.11  			'Pin 7  - B11	
		PORTC = rowA [scancol]			'Green Outputs
		PORTD = rowA [scancol+12]		'White Outputs
		PAUSEUS 500 
		PORTC = 0
		PORTD = 0
		scan = scan * 2 	
	NEXT scancol
	scan = 0									
	INTCON.2 = 0
	RESUME
	ENABLE
END
